package com.vandyo.sentry.core.cases;

import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.vandyo.sentry.core.collectionMachine.CollectionMachine;
import com.vandyo.sentry.core.dto.ErrStatus;
import com.vandyo.sentry.core.dto.Res;
import com.vandyo.sentry.core.exceptions.CaseException;
import com.vandyo.sentry.core.tools.Signature;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

public class CarListDecorator extends CaseDecorator {

    private final static String carListUrl = "/car/my_car/list";

    public CarListDecorator(Case aCase) {
        super(aCase);
    }

    @Override
    public Res<Map<String, String>> check() throws CaseException {
        Res<Map<String, String>> oldRes = super.check();
        if (oldRes.getSuccess()) {
            Map<String,String> data = new HashMap<>();
            data.put("uid",oldRes.getUId());
            try {

                HttpResponse response = HttpRequest.get(Signature.host + carListUrl + "?"
                        + Signature.getUrlParamsByMap(Signature.sign(data, oldRes.getSId())))
                        .execute();
                Res<Map<String, String>> newRes = new Res<>();
                newRes.setSId(oldRes.getSId());
                newRes.setUId(oldRes.getUId());

                if (response.isOk()) {
                    JSONObject json = JSONUtil.parseObj(response.body());
                    JSONArray list = json.getJSONArray("result");
                    if (Objects.isNull(list) || list.size() < 1) {
                        newRes.setSuccess(false);
                        newRes.setStatus(ErrStatus.ErrUnexpected);
                    } else {
                        Map<String, String> carMap = new HashMap<>();
                        JSONObject car = (JSONObject) list.get(0);
                        carMap.put("cid", car.getStr("cid"));
                        newRes.setData(carMap);
                        newRes.setSuccess(true);
                    }
                } else {
                    newRes.setKey(carListUrl);
                    newRes.setStatus(Signature.matchStatus(response.getStatus()));
                    newRes.setSuccess(false);
                }
                CollectionMachine.post(newRes);
                return newRes;
            }catch (Exception e){
                if (e.getMessage().toLowerCase().contains("timeout")){
                    throw new CaseException(carListUrl, ErrStatus.ErrTimeout);
                }else {
                    throw new CaseException(carListUrl,ErrStatus.ErrUnknown);
                }
            }
        } else {
            oldRes.setIgnore(true);
            return oldRes;
        }
    }
}
