package com.vandyo.sentry.core.collectionMachine;

import com.vandyo.sentry.core.Tmp;
import com.vandyo.sentry.core.dto.ErrStatus;
import com.vandyo.sentry.core.dto.Res;
import lombok.Data;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

@Component
public class CollectionMachine {
    private static final Logger logger = LoggerFactory.getLogger(Tmp.class);
    // 收集进来的数据 进到存储 相当于一个临时的日志
    private static Map<Long, Res> temp = new ConcurrentHashMap<>(1024);

    public static void post(Res<Map<String, String>> log) {
        // 应该对于不同的请求统一管理 还是 每个单独管理呢？
//        logger.info("temp long is {}", temp.size());
        if (log.getIgnore() || log.getSuccess()) {
            //pass
        } else {
            if (temp.size() > 100) {
                logger.info("try to put data");
                // 删除一分钟前的key数据
                checkSize(5);
            }
            try {
                temp.put(System.currentTimeMillis(), log);
            } catch (Exception e) {
                logger.error(e.getLocalizedMessage());
            }
        }
        if (temp.size() == 100) {
            System.out.println("Warning!");
        }
    }

    /**
     * 对数据进行归纳
     *
     * @param min 分钟
     * @return
     */
    public static Map<String, Map<ErrStatus, Long>> checkRule(int min) {
        @Data
        class ErrBox {
            private ErrStatus errStatus;
            private Integer count;
        }

        Map<String, List<ErrBox>> map = temp.keySet().stream()
                .filter(x -> x > System.currentTimeMillis() - 1000 * 60 * min)
                .map(x -> temp.get(x))
                .collect(Collectors.toMap(Res::getKey, x -> {
                    List<ErrBox> res = new ArrayList<>();
                    ErrBox box = new ErrBox();
                    box.setCount(1);
                    box.setErrStatus(x.getStatus());
                    res.add(box);
                    return res;
                }, (errStatus, errStatus2) -> {
                    errStatus.addAll(errStatus2);
                    return errStatus;
                }));

        Map<String, Map<ErrStatus, Long>> data = new HashMap<>();

        for (Map.Entry<String, List<ErrBox>> entry : map.entrySet()) {
            data.put(entry.getKey(), entry.getValue().stream().collect(Collectors.groupingBy(ErrBox::getErrStatus, Collectors.counting())));
        }
        logger.info("Get All Err Over!");
        return data;
    }

    public static void checkSize(int min) {
        // 对于超过了时间限制的数据进行删除
        Iterator<Long> iterator = temp.keySet().iterator();
        while (iterator.hasNext()) {
            Long key = iterator.next();
            if (key < System.currentTimeMillis() - 1000 * 60 * min) {
                logger.error("remove key {}", key);
                iterator.remove();
                temp.remove(key);
            } else {
                return;
            }
        }
    }


    /**
     * 检查处理
     *
     * @param timeMap 时间映射
     * @param data    一分钟内错误数据
     * @param dealers 处理器
     */
    public static void checkTimes(Map<String, Integer> timeMap, Map<String, Map<ErrStatus, Long>> data, List<CollectionDealer> dealers) {
        for(Map.Entry<String,Map<ErrStatus, Long>> entry:data.entrySet()){
            String key = entry.getKey();
            if (!timeMap.containsKey(key)){
                continue;
            }
            Map<ErrStatus, Long> value = entry.getValue();
            int sum = 0;
            for(Map.Entry<ErrStatus,Long> i:value.entrySet()){
                sum += i.getValue().intValue();
            }
            if (sum >= timeMap.get(key)){
                //打印 阈值 进行处理
                for (CollectionDealer dealer:dealers){
                    dealer.deal(data);
                }
                break;
            }
        }
    }
}
